@@ -33,6 +33,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #else
 extern char *alloca();
 #endif
+#ifdef USG
+#define index strchr
+#endif
 
 #include "as.h"
 #include "read.h"
@@ -114,7 +117,7 @@ to choose to reach a given label.  */
 #define SIZE_FROM_RELAX_STATE(s) \
   ( (((s) & 0x3) == BYTE ? 1 : (((s) & 0x3) == WORD ? 2 : 4)) )
 
-relax_typeS md_relax_table[] = {
+const relax_typeS md_relax_table[] = {
 /*
   The fields are:
    1) most positive reach of this state,
@@ -159,13 +162,14 @@ void dummy ()
     input_line_pointer++;
 }
 
-pseudo_typeS md_pseudo_table[] = {
+const pseudo_typeS md_pseudo_table[] = {
 	{ "ffloat",	float_cons,	'f' },
 	{ "dfloat",	float_cons,	'd' },
 	{ "tfloat",	float_cons,	'x' },
 	{ "value",      cons,           2 },
 	{ "ident",      dummy,          0   }, /* ignore these directives */
 	{ "def",        dummy,          0   },
+	{ "optim",	dummy,		0   }, /* For sun386i cc */
 	{ "version",    dummy,          0   },
 	{ "ln",    dummy,          0   },
 	{ 0, 0, 0 }
@@ -173,8 +177,8 @@ pseudo_typeS md_pseudo_table[] = {
 
 /* for interface with expression () */
 extern char * input_line_pointer;
-
 char * index ();
+
 char * output_invalid ();
 reg_entry * parse_register ();
 
@@ -282,7 +286,7 @@ void md_begin ()
       
       if (isupper(c) || islower(c) || isdigit(c))
 	operand_chars[c] = c;
-      else if (memchr(operand_special_chars, c, sizeof(operand_special_chars)))
+      else if (c && index(operand_special_chars, c))
 	  operand_chars[c] = c;
       
       if (isdigit(c) || c == '-') digit_chars[c] = c;
@@ -1041,7 +1045,10 @@ void md_assemble (line)
 	   we'll have to start a new one. */
 	/* I caught it failing with obstack_room == 6,
 	   so I changed to <=   pace */
-	if (obstack_room (&frags) <= 6) frag_new (0);
+	if (obstack_room (&frags) <= 6) {
+		frag_wane(frag_now);
+		frag_new (0);
+	}
 	p = frag_more (1);
 	p[0] = t->base_opcode;
 	frag_var (rs_machine_dependent,
@@ -1126,12 +1133,14 @@ void md_assemble (line)
       /* Now the modrm byte and base index byte (if present). */
       if (t->opcode_modifier & Modrm) {
 	p =  frag_more (1);
-	md_number_to_chars (p, i.rm, 1);
+	/* md_number_to_chars (p, i.rm, 1); */
+	md_number_to_chars (p, (i.rm.regmem<<0 | i.rm.reg<<3 | i.rm.mode<<6), 1);
 	/* If i.rm.regmem == ESP (4) && i.rm.mode != Mode 3 (Register mode)
 	   ==> need second modrm byte. */
 	if (i.rm.regmem == ESCAPE_TO_TWO_BYTE_ADDRESSING && i.rm.mode != 3) {
 	  p =  frag_more (1);
-	  md_number_to_chars (p, i.bi, 1);
+	  /* md_number_to_chars (p, i.bi, 1); */
+	  md_number_to_chars (p,(i.bi.base<<0 | i.bi.index<<3 | i.bi.scale<<6), 1);
 	}
       }
       
@@ -1471,6 +1480,8 @@ seg_unimplemented:
       input_line_pointer = displacement_string_start;
       END_STRING_AND_SAVE (displacement_string_end);
       exp_seg = expression (exp);
+      if(*input_line_pointer)
+	as_bad("Ignoring junk '%s' after expression",input_line_pointer);
       RESTORE_END_STRING (displacement_string_end);
       input_line_pointer = save_input_line_pointer;
       switch (exp_seg) {
@@ -1859,7 +1870,7 @@ md_atof(type,litP,sizeP)
   LITTLENUM_TYPE words[MAX_LITTLENUMS];
   LITTLENUM_TYPE *wordP;
   char	*t;
-  char	*atof_i386();
+  char	*atof_ieee();
 
   switch(type) {
   case 'f':
@@ -1881,7 +1892,7 @@ md_atof(type,litP,sizeP)
     *sizeP=0;
     return "Bad call to md_atof ()";
   }
-  t = atof_i386 (input_line_pointer,type,words);
+  t = atof_ieee (input_line_pointer,type,words);
   if(t)
     input_line_pointer=t;
 
